@@ -1,22 +1,24 @@
 /* list.h - Dynamic array and list handling
   
    Copyright (C) 2001 Thomas Moestl
-   Copyright (C) 2002, 2003 Paul A. Rombouts
+   Copyright (C) 2002, 2003, 2007 Paul A. Rombouts
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  This file is part of the pdnsd package.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 
 /* $Id: list.h,v 1.3 2001/06/21 23:58:43 tmm Exp $ */
@@ -28,6 +30,7 @@ Boston, MA 02111-1307, USA.  */
 #include <string.h>
 #include "pdnsd_assert.h"
 
+#if 0
 /*
  * The size of this should always be a multiple of 4 on all supported architectures.
  * Otherwise, we need further glue.
@@ -36,26 +39,29 @@ struct _dynamic_array_dummy_head {
 	unsigned long int nel;	/* number of elements in array */
 	void *elem[0];	/* dummy for alignment */
 };
+#endif
 
-typedef struct _dynamic_array_dummy_head  *darray;
+typedef struct {size_t nel;} *darray;
 
 /* used in type declarations */
-#define DYNAMIC_ARRAY(typ) \
-        struct _dynamic_array_of_ ## typ {unsigned long int nel; typ elem[0]; } 
+#define DYNAMIC_ARRAY(typ)  struct {size_t nel; typ elem[0];} 
+#define DA_OFFSET(a) ((size_t)((typeof (a))0)->elem)
 
-#define DA_CREATE(typ) ((struct _dynamic_array_of_ ## typ *)(da_create(sizeof(typ))))
+/* #define DA_CREATE(typ) ((struct {size_t nel; typ elem[0];} *)(da_create(sizeof(typ)))) */
 #define DA_INDEX(a,i) ((a)->elem[i])
 /* Used often, so make special-case macro here */
 #define DA_LAST(a) ((a)->elem[(a)->nel-1])
 
-#define DA_GROW1(a) ((typeof (a))da_grow1((darray)(a),sizeof((a)->elem[0]),NULL))
-#define DA_GROW1_F(a,cleanup) ((typeof (a))da_grow1((darray)(a),sizeof((a)->elem[0]),cleanup))
-#define DA_RESIZE(a,n) ((typeof (a))da_resize((darray)(a),sizeof((a)->elem[0]),n,NULL))
+#define DA_GROW1(a) ((typeof (a))da_grow1((darray)(a),DA_OFFSET(a),sizeof((a)->elem[0]),NULL))
+#define DA_GROW1_F(a,cleanup) ((typeof (a))da_grow1((darray)(a),DA_OFFSET(a),sizeof((a)->elem[0]),cleanup))
+#define DA_RESIZE(a,n) ((typeof (a))da_resize((darray)(a),DA_OFFSET(a),sizeof((a)->elem[0]),n,NULL))
 #define DA_NEL(a) da_nel((darray)(a))
+
 /*
  * Some or all of these should be inline.
  * They aren't macros for type safety.
  */
+#if 0
 inline static darray Dda_create(size_t sz)
 {
   darray a;
@@ -64,9 +70,10 @@ inline static darray Dda_create(size_t sz)
   if(a) a->nel=0;
   return a;
 }
+#endif
 
-darray da_grow1(darray a, size_t sz, void (*cleanuproutine) (void *));
-darray da_resize(darray a, size_t sz, unsigned int n, void (*cleanuproutine) (void *));
+darray da_grow1(darray a, size_t headsz, size_t elemsz, void (*cleanuproutine) (void *));
+darray da_resize(darray a, size_t headsz, size_t elemsz, size_t n, void (*cleanuproutine) (void *));
 
 inline static unsigned int da_nel(darray a)
 {
@@ -77,13 +84,13 @@ inline static unsigned int da_nel(darray a)
 
 /* alloc/free debug code.*/
 #ifdef ALLOC_DEBUG
-darray DBGda_create(size_t sz, char *file, int line);
-void   DBGda_free(darray a, size_t sz, char *file, int line);
+/* darray DBGda_create(size_t sz, char *file, int line); */
+void   DBGda_free(darray a, size_t headsz, size_t elemsz, char *file, int line);
 
-#define da_create(sz)	DBGda_create(sz, __FILE__, __LINE__)
-#define da_free(a)	DBGda_free((darray)(a),sizeof((a)->elem[0]), __FILE__, __LINE__)
+/* #define da_create(sz)	DBGda_create(sz, __FILE__, __LINE__) */
+#define da_free(a)	DBGda_free((darray)(a),DA_OFFSET(a),sizeof((a)->elem[0]), __FILE__, __LINE__)
 #else
-#define da_create	Dda_create
+/* #define da_create	Dda_create */
 #define da_free		free
 #endif
 
