@@ -1,23 +1,24 @@
 /* rr_types.h - A structure with names & descriptions of
                 all rr types known to pdnsd
    Copyright (C) 2000, 2001 Thomas Moestl
+   Copyright (C) 2007 Paul A. Rombouts
 
-This file is part of the pdnsd package.
+  This file is part of the pdnsd package.
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 /* $Id: rr_types.h,v 1.2 2001/05/09 17:51:52 tmm Exp $ */
 
@@ -67,7 +68,7 @@ Boston, MA 02111-1307, USA.  */
 #define T_NXT      30
 #define T_EID      31
 #define T_NIMLOC   32
-#define T_SRV      33
+#define T_SRV      33    /* additional A*/
 #define T_ATMA     34
 #define T_NAPTR    35
 #define T_KX       36
